'****************************************************************
'*  Name    : I_Schedule_P+.inc                                 *
'*  Author  : John Barrat                                       *
'*  Notice  : Copyright (c) 2004 John Barrat                    *
'*          : All Rights Reserved                               *
'*  Date    : 31/01/2005                                        *
'*  Version : 1.0                                               *
'*  Notes   :                                                   *
'*          :                                                   *
'****************************************************************
'Device = 18F252

' This program provides a basic interrupt scheduling function 
' Requires the presence of an interrupt handler
Symbol INTEDG0  = INTCON2.6   				 ' External Interrupt0 Edge Select bit
Symbol INT0IF	= INTCON.1                   ' INT0 Interrupt Flag

' Scheduler Internal Variables
#IFDEF TASK_DEBUG
Dim PreVal      As Bit
Dim ProCtrl		As Byte
Dim Dummy		As Byte
Dim IdleMax	    As DWord                	 ' Holds the value of a full idle period
Dim IdleCtr     As DWord 	                 ' Maintains a count when Scheduler is idle
#ENDIF

GoTo I_SCHEDULE_Skip

Scheduler_Intr:
' Interrupt Sevice routine for Timed Events
ADDImmd TimedEvents                        	 ' Place TimedEvents on immediate processing list
INTEDG0 = ~ INTEDG0                          ' Set Interrupt to detect opposite edge
Clear INT0IF								 ' Reset the Int 0 Flag
GoTo INT_Code_Cont							 ' Return to Interrupt router

SMALL_MICRO_MODEL = On
InitScheduler:
#IFDEF TASK_DEBUG
    PreVal = SQWPin                          ' Preset the previous value of Squarewave pin
    While PreVal = SQWPin: Wend              ' Wait for first transition of RTC
    PreVal = SQWPin: Clear IdleMax           ' First transition Clear IdleMax Counter
	While PreVal = SQWPin                	 ' Wait for next transition (500MSec)
       Inc IdleMax                 			 ' while incrementing Idle Counter
	Wend
	IdleMax = IdleMax << 1					 ' Multiply by 2 to represent 1 sec
#ENDIF

Scheduler:                                       
#IFDEF TASK_DEBUG
    Inc IdleCtr  							 ' Increment the Idle Counter
#ENDIF
    GoSub Immediate_Tasks
#IFNDEF IOC_HANDLER
    GoSub ScanButtons                         ' Scan Keypad - returns to Scheduler
#ENDIF
    ' Put other background code in here if necessary - If code is lengthy add in additional
    ' GoSub Priority_Tasks to make ensure high priority actions are not missed.
    GoTo Scheduler
    
' Timed Events ***********************************     
TimedEvents:
'   Triggered evey 500mS  Lower frequency triggers are taken from the RTC values.
   GoSub TaskHandler [Rate1]                 ' Do 2Hz Tasks
   If SQWPin = 0 Then                        ' 1Hz Falling Edge detected
        GoSub TaskHandler [Rate2]            ' Do 1Hz Tasks
        If RTCSec = 0 Then                   ' Use the RTC to check for Minute mark
           GoSub GetTimeDate                 ' Get full time & date
           GoSub TaskHandler [Rate3]         ' Do 1/minute Tasks
      	   If RTCMin = 0 Then                ' the hour
       	      GoSub TaskHandler [Rate4]      ' Do this once per hour
      		  If DateChanged = True Then     ' Do this once per day
      	         GoSub TaskHandler [CoD]     ' Call any change of day Tasks   				  
      	         Clear DateChanged           ' Clear DateChanged Flag
              EndIf
           EndIf
        EndIf
#IFDEF TASK_DEBUG
        Clear IdleCtr		   				 ' Reset Idle Counter (1/Sec)
#ENDIF
    EndIf
Return
SMALL_MICRO_MODEL = OFF

#IFDEF TASK_DEBUG
' Report Processor Loading task - run every 1 second *******************************************
' Place on Rate2 task list
SendIdleCtr:
    MsgArry = "^|" + Str$(HEX8 IdleCtr)' + CR + LF
	GoTo SendMessage

EnableProcMon:
    Pop ProCtrl
    If ProCtrl = 0 Then
       REMTask SendIdleCtr, Rate2
    Else
   	   ADDTask SendIdleCtr, Rate2
   	   MsgArry = "^\\" + Str$(HEX8 IdleMax)
	   GoSub SendMessage
    EndIf
Return
#ENDIF
I_SCHEDULE_Skip:
