'****************************************************************
'*  Name    : TASKMAN_INC.pbp                                   *
'*  Author  : John Barrat                                       *
'*  Notice  : Copyright (c) 2004 John Barrat                    *
'*          : All Rights Reserved                               *
'*  Date    : 26/10/2004                                        *
'*  Version : 1.0                                               *
'*  Notes   : Requires the following includes:                  *
'*          :    CON_INC.Bas                                    *
'****************************************************************

' Add the following to the user.mcd file

'KEY ("ADDTask")
'   DeviceCore = [dc16]
'   Highlighter = Keyword 
'   Hint = "<b>ADDTask</b> <i>[<b>Task</b> As Label, <b>Rate</b> As Const], <b>Result</b> As Byte</i>"
'ENDKEY

'KEY ("ADDImmd")
'   DeviceCore = [dc16]
'   Highlighter = Keyword 
'   Hint = "<b>ADDImmd</b> <i>[<b>Task</b> As Label</i>"
'ENDKEY

'KEY ("REMTask")
'   DeviceCore = [dc16]
'   Highlighter = Keyword
'   Hint = "<b>REMTask</b> <i>[<b>Task,</b> As Label, <b>Rate</b> As Const]</i>, <b>Result</b> As Byte</i>"
'ENDKEY

'KEY ("POSTask")
'   DeviceCore = [dc16]
'   Highlighter = Keyword
'   Hint = "<b>POSTask</b> <i>[<b>FirstTask</b> As Label, <b>SecondTask</b> As Label, <b>Rate</b> As Const]</i>, <b>Result</b> As Byte</i>"
'ENDKEY

'KEY ("REPTask")
'   DeviceCore = [dc16]
'   Highlighter = Keyword
'   Hint = "<b>REPTask</b> <i>[<b>CurrentTask</b> As Label, <b>NewTask</b> As Label, <b>Rate</b> As Const]</i>, <b>Result</b> As Byte</i>"
'ENDKEY


'Device = 18F252
Symbol MaxImmdTasks  = ImmdTasks                      ' Maximum number of Interrupt tasks
Symbol ImmdMask		 = ImmdTasks - 1                  ' Set the FIFO rollover mask
Symbol MaxRate1Tasks = Rate1Tasks + MaxImmdTasks      ' Maximum number of tasks every 500mSec
Symbol MaxRate2Tasks = Rate2Tasks + MaxRate1Tasks     ' Maximum number of tasts every Second                
Symbol MaxRate3Tasks = Rate3tasks + MaxRate2Tasks     ' Maximum number of tasks every Minute               
Symbol MaxRate4Tasks = Rate4tasks + MaxRate3Tasks     ' Maximum number of tasks every Hour				
Symbol MaxCoDTasks   = CoDTasks   + MaxRate4Tasks     ' Maximum number of Tasks at Change of Day

' Internal Variables
Dim MaxTasks     As Byte
Dim Tasks[MaxCoDTasks] As Word	  ' Task List
Dim TaskPTR      As Byte	  	  ' Pointer to Current Task
Dim TM_Int       As Byte          ' GP Int Var
Dim TM_Int2      As Byte          ' GP Int Var
Dim Address      As Word          ' Address of Task
Dim Address1     As Word
Dim TM_Len       As Byte   		  ' Length of the Immediate Task List
Dim TM_Head      As Byte   		  ' Pointer to the Start of the Immediate Task List
Dim TM_Tail      As Byte   		  ' Pointer to the End of the Immediate Task list
Dim TM_Stat		 As Byte

' External Control Variables
'Dim ListFull     As Bit
'Dim AlreadyThere As Bit
Dim RptTsks	     As Bit	  		  ' Switch on/off Task list reporting.
Dim RptActTsks	 As Bit			  ' Switch on/off Active Task reporting

Symbol TM_OK        $00
Symbol ListFull     $01
Symbol AlreadyThere $02
Symbol NotFound 	$03

' External Constants:
Symbol RateAll   0
Symbol Rate1     1
Symbol Rate2     2
Symbol Rate3     3
Symbol Rate4     4
Symbol CoD       5

GoTo Skip_TM

' ADD a Task to the list-------------------------------------------------------------------------
' This Macro adds a task to the Task list
' Syntax ADDTask Label, Rate
' Where - Label is the label of the Task to added
'        - Rate is the repetition rate at which the task is to be run 
' It will set ListFull and AlreadyThere booleans before returning.
' It will return with Status byte as follows:
'      0 = OK
'      1 = Already There
'      2 = List Full

ADDTask MACRO P1, P2
#IF (PRM_COUNT >  2)
	#ERROR "ADDTask - Too many parameters"
#ELSE
	#IF (PRM_COUNT <  2)
		#ERROR "ADDTask - Too few parameters"
	#ELSE
		#IF (PRM_1 != LABEL)
			#ERROR "ADDTask - TaskName(Param 1) should be a pointer to a LABEL"
		#ENDIF
		#IF (PRM_2 != Byte) && (PRM_2 != NUM8) && (PRM_2 != NUM16) && (PRM_2 != NUM32)
			#ERROR "ADDTask - Rate(Param 2) should be a Byte variable or number"
		#ENDIF
		#IF (PRM_1 == LABEL)
			NUM_WORD P1, Address
		#ENDIF
		#IF (PRM_2 == Byte)
			BYTE_BYTE P2, TM_Int
		#ENDIF
		#IF (PRM_2 == NUM8)
			NUM_BYTE P2, TM_Int
		#ENDIF
		#IF (PRM_2 == NUM16)
			NUM_BYTE P2, TM_Int
		#ENDIF
		#IF (PRM_2 == NUM32)
			NUM_BYTE P2, TM_Int
		#ENDIF
		GoSub ADDTask_Handler
	#ENDIF
	#IF (ADDTask_RETURN == 1)
		#IF (RETURN_TYPE != Byte)
			#ERROR "ADDTask - Return variable should be a Byte variable"
		#ENDIF
		#IF (RETURN_TYPE == Byte)
			BYTE_BYTE TM_Stat, RETURN_VAR
		#ENDIF
	#ENDIF
#ENDIF
ENDM

' ADD an Interrupt Task to the list--------------------------------------------------------------
'  This Macro adds a high Priority task to the Task list
'  The task will execute immediately the current task has completed
'  and will be executed once only
'  Syntax ADDImmediate Label
'  Where - Label is the label of the Task to added
' ' It will return with Status byte as follows:
'       0 = OK
'       1 = Already There
'       2 = List Full
' 
#IFDEF INT_ENABLED
ADDImmd MACRO P1
#IF (PRM_COUNT >  1)
	#ERROR "ADDImmd - Too many parameters"
#ELSE
	#IF (PRM_COUNT <  1)
		#ERROR "ADDImmd - Too few parameters"
	#ELSE
		#IF (PRM_1 != LABEL)
			#ERROR "ADDImmd - TaskName(Param 1) should be a pointer to a LABEL"
		#ENDIF
		#IF (PRM_1 == LABEL)
			NUM_WORD P1, Address
		#ENDIF
		GoSub ADDImmed_Handler
	#ENDIF
	#IF (ADDImmd_RETURN == 1)
		#IF (RETURN_TYPE != Byte)
			#ERROR "ADDImmd - Return variable should be a Byte variable"
		#ENDIF
		#IF (RETURN_TYPE == Byte)
			BYTE_BYTE TM_Stat, RETURN_VAR
		#ENDIF
	#ENDIF
#ENDIF
ENDM
#ENDIF

' REMove a Task from the list--------------------------------------------------------------------
'  This Macro removes a task from the Task list
'  Syntax REMTask Label, Rate
'  Where - Label is the label of the Task to removerd
'        - Rate is the repetition rate at which the task was being run 
'  It will return with Status byte as follows:
'        0 = OK
'        3 = Not Found if the address is not in the list
' 

REMTask MACRO P1, P2
#IF (PRM_COUNT >  2)
	#ERROR "REMTask - Too many parameters"
#ELSE
	#IF (PRM_COUNT <  2)
		#ERROR "REMTask - Too few parameters"
	#ELSE
		#IF (PRM_1 != LABEL)
			#ERROR "REMTask - TaskName(Param 1) should be a pointer to a LABEL"
		#ENDIF
		#IF (PRM_2 != Byte) && (PRM_2 != NUM8) && (PRM_2 != NUM16) && (PRM_2 != NUM32)
			#ERROR "REMTask - Rate(Param 2) should be a Byte variable or number"
		#ENDIF
		#IF (PRM_1 == LABEL)
			NUM_WORD P1, Address
		#ENDIF
		#IF (PRM_2 == Byte)
			BYTE_BYTE P2, TM_Int
		#ENDIF
		#IF (PRM_2 == NUM8)
			NUM_BYTE P2, TM_Int
		#ENDIF
		#IF (PRM_2 == NUM16)
			NUM_BYTE P2, TM_Int
		#ENDIF
		#IF (PRM_2 == NUM32)
			NUM_BYTE P2, TM_Int
		#ENDIF
		GoSub REMTask_Handler
	#ENDIF
	#IF (REMTask_RETURN == 1)
		#IF (RETURN_TYPE != Byte)
			#ERROR "REMTask - Return variable should be a Byte variable"
		#ENDIF
		#IF (RETURN_TYPE == Byte)
			BYTE_BYTE TM_Stat, RETURN_VAR
		#ENDIF
	#ENDIF
#ENDIF
ENDM

''REMove a Task from the list--------------------------------------------------------------------
'' This Macro removes a task from the Task list
'' Syntax REMTask Label, Rate
'' Where - Label is the label of the Task to removerd
''       - Rate is the repetition rate at which the task was being run 
'' It will return with Status byte as follows:
''       0 = OK
''       3 = Not Found if the address is not in the list

'REMTask MACRO P1 ,P2
'#IF(PRM_COUNT != 2) 
'   #ERROR "REMTask - Incorrect number of parameters"
'#ELSE
'   #IF !(PRM_1 == LABEL)
'       #ERROR "REMTask Param 1 should be a label"
'   #ENDIF
'   #IF ! (PRM_2 == NUM8)
'       #ERROR "REMTask Param 2 should be an 8 bit number"
'   #ENDIF
'   #IF (PRM_1 == LABEL) && (PRM_2 == NUM8)
'        NUM_BYTE P2, TM_Int
'        NUM_WORD P1, Address
'        GoSub REMTask_Handler
'   #ENDIF
'#ENDIF
'ENDM

'  Replaces an existing task with another task
'  Syntax REPTask Label1, Label2, Rate
'  Where - Label 1 is the label of the existing task, 
'  		  Label 2 is the replacement task
' 		  and Rate is the repetition rate the task existing task was being run
'  It will return with Status byte as follows:
'        0 = OK
'        3 = Not Found if the existing address is not in the list

REPTask MACRO P1, P2, P3
#IF (PRM_COUNT >  3)
	#ERROR "REPTask - Too many parameters"
#ELSE
	#IF (PRM_COUNT <  3)
		#ERROR "REPTask - Too few parameters"
	#ELSE
		#IF (PRM_1 != LABEL)
			#ERROR "REPTask - ExitsingTask(Param 1) should be a pointer to a LABEL"
		#ENDIF
		#IF (PRM_2 != LABEL)
			#ERROR "REPTask - NewTask(Param 2) should be a pointer to a LABEL"
		#ENDIF
		#IF (PRM_3 != Byte) && (PRM_3 != NUM8) && (PRM_3 != NUM16) && (PRM_3 != NUM32)
			#ERROR "REPTask - Rate(Param 3) should be a Byte variable or number"
		#ENDIF
		#IF (PRM_1 == LABEL)
			NUM_WORD P1, Address
		#ENDIF
		#IF (PRM_2 == LABEL)
			NUM_WORD P2, Address1
		#ENDIF
		#IF (PRM_3 == Byte)
			BYTE_BYTE P3, TM_Int
		#ENDIF
		#IF (PRM_3 == NUM8)
			NUM_BYTE P3, TM_Int
		#ENDIF
		#IF (PRM_3 == NUM16)
			NUM_BYTE P3, TM_Int
		#ENDIF
		#IF (PRM_3 == NUM32)
			NUM_BYTE P3, TM_Int
		#ENDIF
		GoSub REPTask_Handler
	#ENDIF
	#IF (REPTask_RETURN == 1)
		#IF (RETURN_TYPE != Byte)
			#ERROR "REPTask - Return variable should be a Byte variable"
		#ENDIF
		#IF (RETURN_TYPE == Byte)
			BYTE_BYTE TM_Stat, RETURN_VAR
		#ENDIF
	#ENDIF
#ENDIF
ENDM


'' Replaces an existing task with another task
'' Syntax REPTask Label1, Label2, Rate
'' Where - Label 1 is the label of the existing task, 
'' 		  Label 2 is the replacement task
''		  and Rate is the repetition rate the task existing task was being run
'' It will return with Status byte as follows:
''       0 = OK
''       3 = Not Found if the existing address is not in the list

'REPTask MACRO P1, P2, P3
'#IF(PRM_COUNT != 3) 
'   #ERROR "Incorrect number of parameters in REPTask macro call"
'#ELSE
'   #IF !(PRM_1 == LABEL)
'       #ERROR "Param 1 should be a label"
'   #ENDIF
'   #IF !(PRM_2 == LABEL)
'       #ERROR "Param 2 should be a label"
'   #ENDIF
'   #IF !(PRM_3 == NUM8)
'       #ERROR "Param 2 should be an 8 bit number"
'   #ENDIF
'   #IF (PRM_1 == LABEL) && (PRM_2 == LABEL) && (PRM_3 == NUM8)
'       NUM_BYTE P3, TM_Int
'       NUM_WORD P1, Address
'       NUM_WORD P2, Address1
'       GoSub REPTask_Handler
'   #ENDIF
'#ENDIF
'ENDM

'POSitions a Task relative to another task in the list------------------------------------------
' This macro repositions two tasks relative to each other.
' Syntax POSTask FirstLabel, SecondLabel, Rate
'  Tasks must be present in the list
' will return with AlreadyThere false if either task is not found.  
' If the order is incorrect will swap the positions of the two tasks.
' When called it should pass the following parameters:
'   FirstAddress        The address of the first Task Label
'   SecondAddress       The address of the second Task Label
'   Rate                The rate the tasks were activated
' It will return with Status byte as follows:
'       0 = OK
'       3 = Not Found if either of the addresses are not in the list

POSTask MACRO P1 ,P2, P3
#IF(PRM_COUNT != 3) 
   #ERROR "Incorrect number of parameters in REMTask macro call"
#ELSE
   #IF !(PRM_1 == LABEL)
       #ERROR "Param 1 should be a label"
   #ENDIF
   #IF !(PRM_2 == LABEL)
       #ERROR "Param 2 should be a label"
   #ENDIF
   #IF ! (PRM_3 == NUM8)
       #ERROR "Param 2 should be an 8 bit number"
   #ENDIF
   #IF (PRM_1 == LABEL) && (PRM_2 == LABEL) && (PRM_3 == NUM8)
       NUM_BYTE P3, TM_Int
       NUM_WORD P1, Address
       NUM_WORD P2, Address1
       GoSub POSTask_Handler
   #ENDIF
#ENDIF
ENDM

' Clears down the Tasks Array
SMALL_MICRO_MODEL = On
TaskMan_Init:
    TaskPTR = 0
    MaxTasks = MaxCoDTasks
' Clears all the tasks in the Task list indicated by Set Rate
ClrAllTasks:
 	Clear Tasks                                ' Set all locations in Task array to 0
    TM_Head = 0: TM_Tail = 0: TM_Len = 0   	   ' Reset Immediate FIFO pointers and Length to 0
#IFDEF TASK_DEBUG
    TM_Int = "C"                           	   ' Set result
	RptTsks = False 			  		   	   ' Set Task Reporting to Off
	RptActTsks = False            		   	   ' Set Active Task Reporting to Off
#ENDIF
    GoTo CommonReturn

' This routine points the Task Manager to the set of task rates
' passed with the call  -  Example GoSub Rate [Rate1]                                                                           
Rate:                                    		' External Rate command
   Pop TM_Int                              		' retreive the Rate into TM_Int
RateInt:                                   		' Internal Rate command - TM_Int already set
   TaskPTR  = LookUp TM_Int, [0, MaxImmdTasks,  MaxRate1Tasks, _            
 	                          MaxRate2Tasks, MaxRate3Tasks, MaxRate4Tasks]' Set up the TaskPtr
   MaxTasks = LookUp TM_Int, [MaxCoDTasks,   MaxRate1Tasks, MaxRate2Tasks, _  
                              MaxRate3Tasks, MaxRate4Tasks, MaxCoDTasks]  ' Set up the MaxTask
Return

TaskHandler:
' Task handler will run through all the tasks in the task list for the current Task Rate
' it will return on the first task with an address of 0 or if the task count has reached 
' the max for the current task rate.
GoSub Rate                                  	' Get ptr to start of task list for current rate
Repeat
    Address = Tasks[TaskPTR]                    ' Get the Address of the next task
    If Address = 0 Then Break         			' a 0 address means no further tasks
    Push TaskPTR                                ' save TaskPTR in case current code alters it
	GoSub Execute                               ' Start the task at Address
    Pop TaskPTR                                 ' restore TaskPTR
	Inc TaskPTR                       			' move on down the task list
Until TaskPTR = MaxTasks                    	' and try again
Return

#IFDEF INT_ENABLED
Immediate_Tasks:									
' Immediate_Tasks runs through all Immediate Tasks in task list. Each will only execute once.
' Addresses of Immediate Tasks held in a circular buffer - once executed they must be re-added 
' to the buffer in order to execute again. 
#IFDEF TASK_DEBUG
    TaskPTR = 0									' TaskPTR set to indicate Immediate task run
#ENDIF
While TM_Len > 0 
	Address = Tasks[TM_Head]                    ' Get Address from the Buffer
	Clear Tasks[TM_Head]                        ' Clear head to indicate it has been executed
	Inc TM_Head: Dec TM_Len                     ' Incr pointer to the start of buffer dec Length
	TM_Head = TM_Head & ImmdMask                ' Roll over to 0 if end of buffer
	GoSub Execute                               ' Run task at that address
Wend											' Loop until all entries done
#ENDIF
Return                                      	' return to scheduler

Execute:
' GoTo the address of a Task from a word variable and Return via Task to Handler
#IFDEF TASK_DEBUG
	GoSub SendActTask							' Report the task we are about to run
#ENDIF   
ASM
    movff Address + 1, PCLATH                   ; Set PCH via PCLATH
	movff Address, WREG                         ; Can't write direct to PCL with movff
    movwf PCL                                   ; Writing to PCL will also load PCLath
ENDASM

#IFDEF INT_ENABLED
'ADDImmediate ----------------------------------------------------------------------------------
' Add Immediate Task to Task List.  Sets ListFull if there is no room in buffer
' The Immediate Task List is organised as a FIFO. Tasks will be removed from list on execution
ADDImmed_Handler:
'	Clear AlreadyThere                       	' OK so reset flag
    If Address = Tasks[TM_Head] Then
	   TM_Stat = AlreadyThere                   ' Previous entry not processed set AlreadyThere 
	   Return                                   ' and return
    EndIf	   
	If TM_Len < MaxImmdTasks Then			    ' If Immediate buffer not Full
	   Tasks[TM_Tail] = Address				    ' Add Immediate task Address to the Task list
	   Inc TM_Tail: Inc TM_Len	                ' point to new end of buffer & adjust the length
	   TM_Tail = TM_Tail & ImmdMask             ' Roll over to 0 at end of buffer
	   TM_Stat = TM_OK                          ' Report Status as OK
	Else
	   TM_Stat = ListFull
	EndIf
	#IFDEF TASK_DEBUG
 	TM_Int = "a"
	TaskPTR = 0	  								' TaskPTR set to Immediate Added
	#ENDIF
GoTo CommonReturn
#ENDIF

' Restores Task Pointer ------------------------------------------------------------------------

'FIND a task in the list -----------------------------------------------------------------------
FINDTask:
Clear TM_Stat
   TM_Stat = NotFound                           ' Set Not Found Status
   For TM_Int = TaskPTR To MaxTasks - 1         ' Scan through the List
      If Tasks[TM_Int] = Address Then           ' if Task is in the list?
      	 TM_Stat = AlreadyThere                 ' report Task found
         Break                      		    ' returns with flag set and position in TM_Int
      EndIf
   Next TM_Int
Return

' ADDTask --------------------------------------------------------------------------------------
' This routine adds a task to the Task list 
' It will set ListFull and AlreadyThere booleans before returning.
' It will return the result of the Addition.
ADDTask_Handler:
    GoSub RateInt                               ' Set up the Tasklist Pointers
    GoSub FINDTask                              ' Look for the task in the list
    If TM_Stat = NotFound Then       			' See if it is already there
   	    TM_Stat = ListFull                      ' Its not so set List full (Will reset if found)
		For TM_Int2 = TaskPTR To MaxTasks - 1   ' Look for the first empty slot
	        If Tasks[TM_Int2] = 0 Then          ' Found first empty space
	            Tasks[TM_Int2] = Address        ' add in the new address and
                TM_Stat = TM_OK                 ' and set success boolean
	            Break                 			' exit loop
	        EndIf
	    Next TM_Int2                            ' keep looking for empty slot
    EndIf
#IFDEF TASK_DEBUG
    TM_Int = "A"
#ENDIF
    GoTo CommonReturn

'POSitions a Task relative to another task in the list------------------------------------------
' Positions the tasks in the order requested.  Tasks must already be present in the list
' Returns with TM_Stat OK if both addresses TM_Stat = NotFound if either address not there
POSTask_Handler:
    GoSub RateInt                               ' Set up the Tasklist Pointers
    GoSub FINDTask                              ' Find Address of first task
    If TM_Stat = AlreadyThere Then   			' If first address found
	   TM_Int2 = TM_Int                         ' Save position in TM_Int2
       Swap Address, Address1: GoSub FINDTask   ' Find the Address of second task
       If TM_Stat = AlreadyThere Then   		' If second address found
    ' TM_Int2 holds position of Address and TM_Int holds position of Address1
		  If TM_Int < TM_Int2 Then  			' Test for correct order
          	 Tasks[TM_Int]  = Address1          ' Order incorrect so swap the order
        	 Tasks[TM_Int2] = Address           ' of the addresses
 		  EndIf
		  TM_Stat = TM_OK                       ' Found both so Report OK 
       EndIf
    EndIf
#IFDEF TASK_DEBUG
    TM_Int = "P"
#ENDIF
    GoTo CommonReturn

'REPlace an existing task with a different task-------------------------------------------------
' This routine locates the task defined in Address and replaces it with the task in Address1
' Returns with TM_Stat = TM_OK if done, TM_Stat = NotFound if task not found
REPTask_Handler:
    GoSub RateInt                               ' Set up the Tasklist Pointers
    GoSub FINDTask                              ' Find Address of first task
    If TM_Stat = AlreadyThere Then   			' Try and find address
       Tasks[TM_Int] = Address1                 ' Got it, so Replace the task
	   TM_Stat = TM_OK                          ' and Report OK
	End If
#IFDEF TASK_DEBUG
    TM_Int = "S"
#ENDIF
 	GoTo CommonReturn
	   
'REMove a Task from the list--------------------------------------------------------------------
' This routine removes a task from the current set Rate Task
REMTask_Handler:
    GoSub RateInt                               ' Set up the Tasklist Pointers
    GoSub FINDTask                              ' Find Address of task
	If TM_Stat = AlreadyThere Then           	' Task not found in the List
	    Tasks[TM_Int] = 0                       ' Found it so remove task
	    TM_Int2 = TM_Int                         
	    For TM_Int = TM_Int2 To MaxTasks - 1    ' Shift remaining tasks UP 1 slot
	        Tasks[TM_Int] = Tasks[TM_Int + 1]   ' copy next Slot to current Slot
	        If Tasks[TM_Int + 1] = 0 Then       ' if next Slot is empty?
	            GoTo ShiftDone                  ' shift is finished
	        EndIf
	    Next TM_Int
ShiftDone:
	    Tasks[MaxTasks - 1] = 0                 ' Clear the last slot in the List
	    TM_Stat = TM_OK                         ' Report OK
    EndIf
#IFDEF TASK_DEBUG
    TM_Int = "R"
#ENDIF
'Common exit routine for ADD, REM, POS, REP and Clear-------------------------------------------
CommonReturn:
#IFNDEF TASK_DEBUG
    Return
#ELSE

' Debug code - prints out the addresses held in the task list.**********************************
If RptTsks = False Then Return 
'If TM_Int = "E" Then Return
ReportTasks:
' Outputs 'Active Tasks Addresses' on the Serial port for most recently changed Rate in the form
'   TaskPTR Value, Status, Action (A=Add, R=remove, P=reposition, S=Replace)
'   Dec Address, {Dec Address} ....... CR LF'
   MsgArry = "^RTK," + Str$(DEC2 TaskPTR) + "," + Str$(HEX2 TM_Stat) + "," + TM_Int + "," + Str$(HEX4 Address)
   If Address1 > 0 Then 
   	  MsgArry = MsgArry  + "," + Str$(HEX4 Address1)
   	  Address1 = 0
   EndIf
   GoTo SendDebugMsg

SendActTask:
' Outputs the Current Rate in task list and address of the task about to be executed
   If RptActTsks = False Then Return
   MsgArry = "^RAT," + Str$(DEC2 TaskPTR) + "," + Str$(HEX4 Address)
   GoTo SendDebugMsg

' This routine returns a list of task pointer start addresses for each task rate
' It assumes that task rate 1 starts at 0.  Use this to determine the start locations in the
' list for each Task Rate and the max number of tasks for each rate.
SendRateLocs:
   MsgArry = "^RTR," + Str$(DEC2 MaxImmdTasks) + "," + Str$(Dec MaxRate1Tasks) + "," + Str$(Dec MaxRate2Tasks) + "," + Str$(Dec MaxRate3Tasks) + "," + Str$(Dec MaxRate4Tasks) + "," + Str$(Dec MaxCoDTasks)	     

SendDebugMsg:
SMALL_MICRO_MODEL = Off
   GoTo SendMessage

'This routine sends out a snapshot of the whole task table. Use this to prime the initial state
'SMALL_MICRO_MODEL = On
SendAllTasks:
TM_Int = "I": Clear TM_Stat
TaskPTR = ImmdTasks
Repeat
   Address = Tasks[TaskPTR]
   If Address > 0 Then 
   	  Address1 = 0
   	  GoSub ReportTasks
   EndIf
   Inc TaskPTR
Until TaskPTR = MaxCoDTasks -1 
Return
'SMALL_MICRO_MODEL = Off

#ENDIF  
Skip_TM:
